"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const CompilerOptions_1 = require("../../CompilerOptions");
const lua = require("../../LuaAST");
const errors_1 = require("../utils/errors");
const scope_1 = require("../utils/scope");
exports.transformBreakStatement = (breakStatement, context) => {
    const breakableScope = scope_1.findScope(context, scope_1.ScopeType.Loop | scope_1.ScopeType.Switch);
    if (breakableScope === undefined) {
        throw errors_1.UndefinedScope();
    }
    if (breakableScope.type === scope_1.ScopeType.Switch) {
        return lua.createGotoStatement(`____switch${breakableScope.id}_end`);
    }
    else {
        return lua.createBreakStatement(breakStatement);
    }
};
exports.transformContinueStatement = (statement, context) => {
    if (context.luaTarget === CompilerOptions_1.LuaTarget.Lua51) {
        throw errors_1.UnsupportedForTarget("Continue statement", CompilerOptions_1.LuaTarget.Lua51, statement);
    }
    const scope = scope_1.findScope(context, scope_1.ScopeType.Loop);
    if (scope === undefined) {
        throw errors_1.UndefinedScope();
    }
    scope.loopContinued = true;
    return lua.createGotoStatement(`__continue${scope.id}`, statement);
};
//# sourceMappingURL=break-continue.js.map