"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript");
const lua = require("../../../LuaAST");
const scope_1 = require("../../utils/scope");
const block_1 = require("../block");
const variable_declaration_1 = require("../variable-declaration");
function transformLoopBody(context, loop) {
    scope_1.pushScope(context, scope_1.ScopeType.Loop);
    const body = scope_1.performHoisting(context, block_1.transformBlockOrStatement(context, loop.statement));
    const scope = scope_1.popScope(context);
    const scopeId = scope.id;
    if (!scope.loopContinued) {
        return body;
    }
    const baseResult = [lua.createDoStatement(body)];
    const continueLabel = lua.createLabelStatement(`__continue${scopeId}`);
    baseResult.push(continueLabel);
    return baseResult;
}
exports.transformLoopBody = transformLoopBody;
function getVariableDeclarationBinding(node) {
    variable_declaration_1.checkVariableDeclarationList(node);
    if (node.declarations.length === 0) {
        return ts.createIdentifier("____");
    }
    return node.declarations[0].name;
}
exports.getVariableDeclarationBinding = getVariableDeclarationBinding;
//# sourceMappingURL=utils.js.map